@@ -205,7 +205,7 @@ nvme_list_opts () {
 		"get-feature")
 		opts+=" --namespace-id= -n --feature-id= -f --sel= -s \
 			--data-len= -l --cdw11= --c -uuid-index= -U --raw-binary -b \
-			--human-readable -H --timeout= -t"
+			--human-readable -H --timeout= -t --changed -C"
 			;;
 		"device-self-test")
 		opts+=" --namespace-id= -n --self-test-code= -s --timeout= -t"
@@ -356,10 +356,11 @@ nvme_list_opts () {
 			;;
 		"sanitize")
 		opts+=" --no-dealloc -d --oipbp -i --owpass= -n \
-			--ause -u --sanact= -a --ovrpat= -p"
+			--ause -u --sanact= -a --ovrpat= -p --emvs= -e"
 		case $opt in
 			--sanact|-a)
-			vals+=" exit-failure start-block-erase start-overwrite start-crypto-erase"
+			vals+=" exit-failure start-block-erase start-overwrite \
+				start-crypto-erase exit-media-verification"
 				;;
 		esac
 			;;
@@ -1181,6 +1182,13 @@ plugin_solidigm_opts () {
 		"temp-stats")
 		opts+=" --raw-binary -b"
 			;;
+		"workload-tracker")
+		opts+=" --enable -e --disable -d --sample-time= -s \
+		--type= -t --run-time= -r --flush-freq= -f \
+		--wall-clock -w --trigger-field= -T \
+		--trigger-threshold= -V --trigger-on-delta -D \
+		--trigger-on-latency -L --verbose -v"
+			;;
 		"version")
 		opts+=$NO_OPTS
 			;;
@@ -1228,6 +1236,38 @@ plugin_transcend_opts () {
 	return 0
 }
 
+plugin_dapustor_opts () {
+	local opts=""
+	local compargs=""
+
+	local nonopt_args=0
+	for (( i=0; i < ${#words[@]}-1; i++ )); do
+		if [[ ${words[i]} != -* ]]; then
+			let nonopt_args+=1
+		fi
+	done
+
+	if [ $nonopt_args -eq 3 ]; then
+		opts="/dev/nvme* "
+	fi
+
+	opts+=" "
+
+	case "$1" in
+		"smart-log-add")
+		opts+=" --namespace-id= -n --raw-binary -b \
+			--json -j"
+			;;
+		"help")
+		opts+=$NO_OPTS
+			;;
+	esac
+
+	COMPREPLY+=( $( compgen $compargs -W "$opts" -- $cur ) )
+
+	return 0
+}
+
 plugin_zns_opts () {
 	local opts=""
 	local compargs=""
@@ -1491,6 +1531,10 @@ plugin_ocp_opts () {
 		opts+=" --data= -d --number= -n --no-uuid -N --type= -t \
 			--nrtdp= -r --verbose -v --output-format -o --timeout="
 			;;
+		"hardware-component-log")
+		opts+=" --comp-id= -i --list -l --verbose -v \
+			--output-format -o --timeout= -t"
+			;;
 		"help")
 		opts+=$NO_OPTS
 			;;
@@ -1552,8 +1596,9 @@ _nvme_subcmds () {
 			clear-pcie-correctable-errors parse-telemetry-log \
 			clear-fw-activate-history vs-fw-activate-history log-page-directory \
 			vs-drive-info cloud-SSDplugin-version market-log \
-			smart-log-add temp-stats version help"
+			smart-log-add temp-stats workload-tracker version help"
 		[transcend]="healthvalue badblock"
+		[dapustor]="smart-log-add"
 		[zns]="id-ctrl id-ns zone-mgmt-recv \
 			zone-mgmt-send report-zones close-zone \
 			finish-zone open-zone reset-zone offline-zone \
@@ -1569,7 +1614,8 @@ _nvme_subcmds () {
 			set-dssd-power-state-feature get-dssd-power-state-feature \
 			telemetry-string-log set-telemetry-profile \
 			set-dssd-async-event-config get-dssd-async-event-config \
-			get-error-injection set-error-injection"
+			get-error-injection set-error-injection \
+			hardware-component-log"
 	)
 
 	# Associative array mapping plugins to corresponding option completions
@@ -1588,6 +1634,7 @@ _nvme_subcmds () {
 		[sfx]="plugin_sfx_opts"
 		[solidigm]="plugin_solidigm_opts"
 		[transcend]="plugin_transcend_opts"
+		[dapustor]="plugin_dapustor_opts"
 		[zns]="plugin_zns_opts"
 		[nvidia]="plugin_nvidia_opts"
 		[ymtc]="plugin_ymtc_opts"
