@@ -47,7 +47,6 @@ enum argconfig_types {
 	CFG_FLAG,
 	CFG_STRING,
 	CFG_INT,
-	CFG_SIZE,
 	CFG_LONG,
 	CFG_LONG_SUFFIX,
 	CFG_DOUBLE,
@@ -181,7 +180,6 @@ int argconfig_parse_comma_sep_array_u32(char *string, __u32 *val,
 					unsigned int max_length);
 int argconfig_parse_comma_sep_array_u64(char *string, __u64 *val,
 					unsigned int max_length);
-int argconfig_parse_byte(const char *opt, const char *str, unsigned char *val);
 
 void print_word_wrapped(const char *s, int indent, int start, FILE *stream);
 bool argconfig_parse_seen(struct argconfig_commandline_options *options,
