@@ -5,6 +5,12 @@
 
 #include <stdbool.h>
 
+#define print_info(...) \
+	do { \
+		if (log_level >= LOG_INFO) \
+			printf(__VA_ARGS__); \
+	} while (false)
+
 extern int log_level;
 
 int map_log_level(int verbose, bool quiet);
